<?php

namespace Drupal\Tests\commerce_order_item_sku\Kernel;

use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_order\Entity\OrderItemType;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\Tests\commerce_order\Kernel\OrderKernelTestBase;

/**
 * Tests the commerce_order_item_sku_entity_update() function.
 *
 * @group commerce_order_item_sku
 */
class PurchasableEntityUpdateTest extends OrderKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['commerce_order_item_sku'];

  /**
   * The queue.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * The queue worker.
   *
   * @var \Drupal\Core\Queue\QueueWorkerInterface
   */
  protected $queueWorker;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['commerce_order_item_sku']);

    $this->configFactory = $this->container->get('config.factory');
    $this->configFactory->getEditable('commerce_order_item_sku.settings')
      ->set('setting_event', 'order_process')
      ->set('sync_on_purchasable_entity_sku_update', FALSE)
      ->set('sync_on_purchased_entity_reference_update', FALSE)
      ->save();

    $order_item_type = OrderItemType::load('test');
    $order_item_type->setTraits(['commerce_order_item_sku']);
    $order_item_type->save();

    $trait_manager = $this->container->get('plugin.manager.commerce_entity_trait');
    $trait = $trait_manager->createInstance('commerce_order_item_sku');
    $trait_manager->installTrait($trait, 'commerce_order_item', 'test');

    $variation = ProductVariation::create([
      'type' => 'default',
      'sku' => 'ORIGINAL_SKU',
    ]);
    $variation->save();

    $order_item = OrderItem::create([
      'type' => 'test',
      'purchased_entity' => $variation->id(),
      'sku' => 'ORIGINAL_SKU',
    ]);
    $order_item->save();

    $this->queue = $this->container->get('queue')->get('commerce_order_item_sku');
    $this->queueWorker = $this->container->get('plugin.manager.queue_worker')->createInstance('commerce_order_item_sku');
  }

  /**
   * Tests the entity update.
   */
  public function testEntityUpdate() {
    $variation = ProductVariation::load(1);
    $variation->set('sku', 'NOT_SYNC_SKU');
    $variation->save();

    $this->queue->createQueue();
    $item = $this->queue->claimItem();
    $this->assertFalse(
      $item,
      'No queue item should have been created for non-synced SKU.'
    );

    $variation->set('sku', 'ORIGINAL_SKU');
    $variation->save();

    $this->configFactory->getEditable('commerce_order_item_sku.settings')
      ->set('sync_on_purchasable_entity_sku_update', TRUE)
      ->save();

    $variation->set('sku', 'SYNC_SKU');
    $variation->save();

    $this->queue->createQueue();
    $item = $this->queue->claimItem();
    $this->assertNotFalse(
      $item,
      'A queue item should have been created for synced SKU.'
    );
    $this->queueWorker->processItem($item->data);

    $order_item = OrderItem::load(1);
    $this->assertEquals(
      'SYNC_SKU',
      $order_item->get('sku')->value,
      'The order item SKU should be updated after the variation SKU is updated.'
    );

    $order_item->set('sku', 'UPDATED_SKU');
    $order_item->save();

    $variation->set('sku', 'NEW_SKU');
    $variation->save();

    $this->queue->createQueue();
    $item = $this->queue->claimItem();
    $this->assertNotFalse(
      $item,
      'A queue item should have been created for synced SKU.'
    );
    $this->queueWorker->processItem($item->data);

    $order_item = OrderItem::load(1);
    $this->assertEquals(
      'UPDATED_SKU',
      $order_item->get('sku')->value,
      'The order item SKU should not be updated after it was updated.'
    );
  }

}
